/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "gallerynative.h"

gboolean request_img(int x);
gboolean request_service_connection(void);

struct priv {
	sap_agent_h agent;
	sap_socket_h socket;
	sap_peer_agent_h peer_agent;
};

typedef struct appData {
	const char *msgId;
	char *id;
	const char *result;
	int reason;
	int count;
	GList *list;
} data;

static data testData = { 0 };
static GList *Imagelist = NULL;
static struct priv priv_data = { 0 };
gboolean peer_connected = FALSE;

static char encoding_table[] = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
				 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
				 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
				 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
				 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
				 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
				 'w', 'x', 'y', 'z', '0', '1', '2', '3',
				 '4', '5', '6', '7', '8', '9', '+', '/' };
static char *decoding_table = NULL;
static int mod_table[] = { 0, 2, 1 };

void build_decoding_table()
{
	int i;
	decoding_table = malloc(256);

	for (i = 0; i < 64; i++)
		decoding_table[(unsigned char)encoding_table[i]] = i;
}

char *base64_encode(const unsigned char *data, size_t input_length, size_t *output_length)
{
	*output_length = 4 * ((input_length + 2) / 3);

	char *encoded_data = malloc(*output_length + 3);
	if (encoded_data == NULL) return NULL;

	int i, j;
	for (i = 0, j = 0; i < input_length; ) {
		uint32_t octet_a = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t octet_b = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t octet_c = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t triple = (octet_a << 0x10) + (octet_b << 0x08) + octet_c;

		encoded_data[j++] = encoding_table[(triple >> 3 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 2 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 1 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 0 * 6) & 0x3F];
	}

	for (i = 0; i < mod_table[input_length % 3]; i++) encoded_data[*output_length - 1 - i] = '=';

	encoded_data[*output_length] = '\\';
	encoded_data[*output_length + 1] = 'n';
	encoded_data[*output_length + 2] = '\0';
	return encoded_data;

}

unsigned char *base64_decode(const char *data, size_t input_length, size_t *output_length)
{
	if (decoding_table == NULL) build_decoding_table();

	if (input_length % 4 != 0) return NULL;

	*output_length = input_length / 4 * 3;
	if (data[input_length - 1] == '=') (*output_length)--;
	if (data[input_length - 2] == '=') (*output_length)--;

	unsigned char *decoded_data = malloc(*output_length);
	if (decoded_data == NULL) return NULL;

	int i, j;

	for (i = 0, j = 0; i < input_length; ) {
		uint32_t sextet_a = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
		uint32_t sextet_b = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
		uint32_t sextet_c = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
		uint32_t sextet_d = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];

		uint32_t triple = (sextet_a << 3 * 6)
				  + (sextet_b << 2 * 6)
				  + (sextet_c << 1 * 6)
				  + (sextet_d << 0 * 6);

		if (j < *output_length) decoded_data[j++] = (triple >> 2 * 8) & 0xFF;
		if (j < *output_length) decoded_data[j++] = (triple >> 1 * 8) & 0xFF;
		if (j < *output_length) decoded_data[j++] = (triple >> 0 * 8) & 0xFF;
	}

	return decoded_data;
}

char *compose_list_request_msg(data *ad)
{
	dlog_print(DLOG_ERROR, TAG, "Comsposing list request message");
	/*Initialization*/
	JsonGenerator *jsonGenerator = json_generator_new();
	JsonObject *jMainObject = json_object_new();
	JsonNode *jNode = json_node_new(JSON_NODE_OBJECT);

	json_object_set_string_member(jMainObject, MSG_ID, ad->msgId);
	json_object_set_string_member(jMainObject, OFFSET, ad->id);

	unsigned int objSize = json_object_get_size(jMainObject);
	json_node_set_object(jNode, jMainObject);
	json_generator_set_root(jsonGenerator, jNode);

	char *jsonObj = json_generator_to_data(jsonGenerator, &objSize);

	json_node_free(jNode);
	return jsonObj;
}

char *compose_image_req_msg(image *ad)
{
	dlog_print(DLOG_INFO, TAG, "Comsposing img resp");

	JsonGenerator *jsonGenerator = json_generator_new();
	JsonObject *jMainObject = json_object_new();
	JsonNode *jNode = json_node_new(JSON_NODE_OBJECT);
	ad->msgId = DOWNSCALE_IMG_REQ;
	char id[100];
	sprintf(id, "%d", ad->id);

	json_object_set_string_member(jMainObject, MSG_ID, ad->msgId);
	json_object_set_int_member(jMainObject, IMAGE_ID, ad->id);
	json_object_set_int_member(jMainObject, WIDTH, 320);
	json_object_set_int_member(jMainObject, HEIGHT, 320);

	unsigned int objSize = json_object_get_size(jMainObject);
	json_node_set_object(jNode, jMainObject);
	json_generator_set_root(jsonGenerator, jNode);

	char *jsonObj = json_generator_to_data(jsonGenerator, &objSize);

	json_node_free(jNode);
	return jsonObj;
}

gchar *ExtractValue(JsonNode *node)
{
	gchar *value;
	GType valueType = json_node_get_value_type(node);
	switch (valueType) {
	case G_TYPE_STRING:
		value = json_node_dup_string(node);
		break;

	case G_TYPE_INT:
		value = malloc(20 * sizeof(gchar));
		sprintf(value, "%d", (int)json_node_get_int(node));
		break;

	case G_TYPE_DOUBLE:
		value = malloc(40 * sizeof(gchar));
		sprintf(value, "%f", json_node_get_double(node));
		break;

	case G_TYPE_BOOLEAN:
		value = malloc(6 * sizeof(gchar));
		if (json_node_get_boolean(node) == TRUE) {
			sprintf(value, "%s", "true");
		} else {
			sprintf(value, "%s", "false");
		}
		break;

	default:
		value = malloc(22 * sizeof(gchar));
		sprintf(value, "%s", "Value of unknown type");
		break;
	}
	return value;
}

void parse_image_res(JsonObject *jResponseData)
{
	JsonObject *jData;
	size_t out = 0;
	const char *result;
	result = json_object_get_string_member(jResponseData, "result");

	if (strcmp(result, "failure")) {

		jData = json_object_get_object_member(jResponseData, IMAGE);

		image *imageInfo = NULL;
		imageInfo = g_try_new0(struct image, 1);

		imageInfo->data = json_object_get_string_member(jData, IMAGE);
		imageInfo->id = json_object_get_int_member(jData, IMAGE_ID);
		imageInfo->width = json_object_get_string_member(jData, WIDTH);
		imageInfo->height = json_object_get_string_member(jData, HEIGHT);

		imageInfo->data = (char *)base64_decode(imageInfo->data, (size_t)strlen(imageInfo->data), &out);
		imageInfo->size = out;
		display_image((void *)imageInfo->data, imageInfo->size, (int)imageInfo->width, (int)imageInfo->height);
	} else {
		request_list_msg(-1);
	}
}

void parse_list_res(JsonObject *jResponseData)
{
	JsonArray *jDataArray;
	GList *temp_lis = NULL;

	jDataArray = json_object_get_array_member(jResponseData, "list");
	testData.result = json_object_get_string_member(jResponseData, "result");
	dlog_print(DLOG_INFO, TAG, "the result is %s", testData.result);
	if (strcmp(testData.result, "failure")) {
		int count = json_array_get_length(jDataArray);
		for (int i = 0; i < count; i++) {
			size_t out = 0;
			image *imageInfo = NULL;
			gboolean imgExists = FALSE;
			imageInfo = g_try_new0(struct image, 1);

			JsonObject *jData = json_array_get_object_element(jDataArray, i);
			imageInfo->data = json_object_get_string_member(jData, IMAGE);
			imageInfo->id = json_object_get_int_member(jData, IMAGE_ID);
			imageInfo->name = json_object_get_string_member(jData, IMAGE_NAME);

			imageInfo->data = (const char *)base64_decode(imageInfo->data, (size_t)strlen(imageInfo->data), &out);
			imageInfo->size = out;
			temp_lis = Imagelist;
			while (temp_lis) {

				image *im = { 0 };
				im = (image *)temp_lis->data;
				if (im->id == imageInfo->id) {
					imgExists = TRUE;
					break;
				}
				temp_lis = temp_lis->next;
			}
			if (!imgExists)
				Imagelist = g_list_append(Imagelist, imageInfo);
		}
	}
	display_thumbnail_list(Imagelist);
}


void on_peer_agent_updated(sap_peer_agent_h peer_agent,
			   sap_peer_agent_status_e peer_status,
			   sap_peer_agent_found_result_e result,
			   void *user_data)
{
	switch (result) {

	case SAP_PEER_AGENT_FOUND_RESULT_DEVICE_NOT_CONNECTED:

		dlog_print(DLOG_ERROR, TAG, "device is not connected");

		break;

	case SAP_PEER_AGENT_FOUND_RESULT_FOUND:

		if (peer_status == SAP_PEER_AGENT_STATUS_AVAILABLE) {
			priv_data.peer_agent = peer_agent;
			request_service_connection();
		} else {
			dlog_print(DLOG_ERROR, TAG, "peer agent removed");
			sap_peer_agent_destroy(peer_agent);
		}
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_SERVICE_NOT_FOUND:

		dlog_print(DLOG_ERROR, TAG, "service not found");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_TIMEDOUT:

		dlog_print(DLOG_ERROR, TAG, "peer agent find timed out");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_INTERNAL_ERROR:

		dlog_print(DLOG_ERROR, TAG, "peer agent find search failed");
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", peer_status);
		break;

	}
}

static void on_data_recieved(sap_socket_h socket,
			     unsigned short int channel_id,
			     unsigned int payload_length,
			     void *buffer,
			     void *user_data)
{
	dlog_print(DLOG_INFO, TAG, "received data: %s, len:%d", buffer, payload_length);
	JsonNode *root;

	JsonParser *parser = json_parser_new();
	json_parser_load_from_data(parser, buffer, payload_length, NULL);

	JsonObject *jResponseData;
	root = json_parser_get_root(parser);
	if (NULL != root) {
		jResponseData = json_node_get_object(root);
		testData.msgId = json_object_get_string_member(jResponseData, "msgId");

		if (strcmp(testData.msgId, THUMBNAIL_LIST_RSP) == 0) {
			parse_list_res(jResponseData);
		} else if (strcmp(testData.msgId, DOWNSCALE_IMG_RSP) == 0) {
			parse_image_res(jResponseData);
		}
	}
}

void request_list_msg(int id)
{
	char *data, buf[100];
	int result;
	testData.msgId = THUMBNAIL_LIST_REQ;
	sprintf(buf, "%d", id);
	testData.id = buf;
	data = compose_list_request_msg(&testData);
	if (priv_data.socket) {
		result = sap_socket_send_data(priv_data.socket, GALLERY_CHANNELID, strlen(data), data);
	}
}

static void on_service_connection_created(sap_peer_agent_h peer_agent,
					  sap_socket_h socket,
					  sap_service_connection_result_e result,
					  void *user_data)
{
	switch (result) {

	case SAP_CONNECTION_SUCCESS:
		peer_connected = TRUE;
		dlog_print(DLOG_DEBUG, TAG, "peer agent connection is successful, pa :%u", peer_agent);
		sap_socket_set_data_received_cb(socket, on_data_recieved, peer_agent);
		priv_data.socket = socket;
		request_list_msg(-1);
		break;

	case SAP_CONNECTION_ALREADY_EXIST:
		peer_connected = TRUE;
		dlog_print(DLOG_DEBUG, TAG, "connection is already exist");
		priv_data.socket = socket;
		break;

	case SAP_CONNECTION_FAILURE_DEVICE_UNREACHABLE:
		dlog_print(DLOG_ERROR, TAG, "device is not unreachable");
		break;

	case SAP_CONNECTION_FAILURE_INVALID_PEERAGENT:
		dlog_print(DLOG_ERROR, TAG, "invalid peer agent");
		break;

	case SAP_CONNECTION_FAILURE_NETWORK:
		dlog_print(DLOG_ERROR, TAG, "network failure");
		break;

	case SAP_CONNECTION_FAILURE_PEERAGENT_NO_RESPONSE:
		dlog_print(DLOG_ERROR, TAG, "peer agent is no response");
		break;

	case SAP_CONNECTION_FAILURE_PEERAGENT_REJECTED:
		dlog_print(DLOG_ERROR, TAG, "peer agent is rejected");
		break;

	case SAP_CONNECTION_IN_PROGRESS:
		dlog_print(DLOG_ERROR, TAG, "connection is in progress");
		break;

	case SAP_CONNECTION_PEER_AGENT_NOT_SUPPORTED:
		dlog_print(DLOG_ERROR, TAG, "peer agent not supported");
		break;

	case SAP_CONNECTION_FAILURE_UNKNOWN:
		dlog_print(DLOG_ERROR, TAG, "sap conn failure unknown");
		break;
	}
}

static gboolean _create_service_connection(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	priv = (struct priv *)user_data;

	result = sap_agent_request_service_connection(priv->agent,
						      priv->peer_agent,
						      on_service_connection_created,
						      NULL);

	if (result == SAP_RESULT_SUCCESS) {
		dlog_print(DLOG_DEBUG, TAG, "req service conn call succeeded");
		return TRUE;
	} else {
		dlog_print(DLOG_ERROR, TAG, "req service conn call is failed (%d)", result);
		return FALSE;
	}
}

gboolean request_service_connection(void)
{
	if (peer_connected == FALSE) {
		dlog_print(DLOG_ERROR, TAG, "requesting conn.");
		_create_service_connection(&priv_data);
	}

	return TRUE;
}

gboolean terminate_service_connection(void)
{
	//g_idle_add(_terminate_service_connection, &priv_data);
	return TRUE;
}

static gboolean _find_peer_agent(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	priv = (struct priv *)user_data;

	result = sap_agent_find_peer_agent(priv->agent, on_peer_agent_updated, NULL);

	if (result == SAP_RESULT_SUCCESS) {
		dlog_print(DLOG_DEBUG, TAG, "find peer call succeeded");
	} else {
		dlog_print(DLOG_ERROR, TAG, "findsap_peer_agent_s is failed (%d)", result);
	}

	dlog_print(DLOG_INFO, TAG, "find peer call is over");

	return FALSE;
}

gboolean find_peers()
{
	if (peer_connected == FALSE) {
		_find_peer_agent(&priv_data);
	}
	return TRUE;
}


gboolean request_img(int index)
{
	int result;
	char *new_message = "";
	GList *temp_list = NULL;
	if (priv_data.socket) {

		temp_list = Imagelist;
		while (temp_list) {
			image *img = { 0 };
			img = (image *)temp_list->data;
			if (img->id == index) {
				new_message = compose_image_req_msg(img);
				result = sap_socket_send_data(priv_data.socket, GALLERY_CHANNELID, strlen(new_message), new_message);
				break;
			}
			temp_list = temp_list->next;
		}

	}

	return TRUE;
}

static void on_agent_initialized(sap_agent_h agent,
				 sap_agent_initialized_result_e result,
				 void *user_data)
{
	switch (result) {
	case SAP_AGENT_INITIALIZED_RESULT_SUCCESS:
		dlog_print(DLOG_DEBUG, TAG, "agent is initialized");
		priv_data.agent = agent;
		break;

	case SAP_AGENT_INITIALIZED_RESULT_DUPLICATED:
		dlog_print(DLOG_ERROR, TAG, "duplicate registration");

		break;

	case SAP_AGENT_INITIALIZED_RESULT_INVALID_ARGUMENTS:
		dlog_print(DLOG_ERROR, TAG, "invalid arguments");

		break;

	case SAP_AGENT_INITIALIZED_RESULT_INTERNAL_ERROR:
		dlog_print(DLOG_ERROR, TAG, "internal sap error");

		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", result);

		break;
	}

	dlog_print(DLOG_DEBUG, TAG, "agent initialized callback is over");

}

static void on_device_status_changed(sap_device_status_e status, sap_transport_type_e transport_type,
				     void *user_data)
{
	switch (status) {
	case SAP_DEVICE_STATUS_DETACHED:

		if (priv_data.peer_agent) {
			sap_peer_agent_destroy(priv_data.peer_agent);
			priv_data.peer_agent = NULL;
			sap_socket_destroy(priv_data.socket);
			priv_data.socket = NULL;
		}

		peer_connected = 0;
		break;

	case SAP_DEVICE_STATUS_ATTACHED:
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", status);
		break;
	}

}

gboolean agent_initialize()
{
	int result = 0;

	do {
		result = sap_agent_initialize(priv_data.agent, GALLERY_ASPID, SAP_AGENT_ROLE_CONSUMER,
					      on_agent_initialized, NULL);

		dlog_print(DLOG_INFO, TAG, "SAP >>> getRegisteredServiceAgent() >>> %d", result);
	} while (result != SAP_RESULT_SUCCESS);

	return TRUE;
}

void initialize_sap(void)
{
	sap_agent_h agent = NULL;
	testData.list = NULL;

	sap_agent_create(&agent);

	if (agent == NULL)
		dlog_print(DLOG_ERROR, TAG, "ERROR in creating agent");
	else
		dlog_print(DLOG_INFO, TAG, "SUCCESSFULLY create sap agent");

	priv_data.agent = agent;
	sap_set_device_status_changed_cb(on_device_status_changed, NULL);
	agent_initialize();
}
