/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <stdio.h>

#include <dlog.h>
#include "composer.h"

char *addJSONListStr(char *jStr, char *k, char *v)
{
	int nj = 0;

	nj = (int)strlen(jStr);
	if (nj == 2) {
		jStr[strlen(jStr) - 1] = '\0';
		//jSize--;
	}
	if (nj > 2) {
		jStr[strlen(jStr) - 1] = ',';
	}
	strcat(jStr, "\"");
	strcat(jStr, k);
	strcat(jStr, "\":[");

	strcat(jStr, v);

	return jStr;
}

char *addImageJSON(char *jStr, char *k, char *v)
{
	int nj = 0;

	nj = (int)strlen(jStr);
	if (nj == 2) {
		jStr[strlen(jStr) - 1] = '\0';
		//jSize--;
	}
	if (nj > 2) {
		jStr[strlen(jStr) - 1] = ',';
	}
	strcat(jStr, "\"");
	strcat(jStr, k);
	strcat(jStr, "\":");
	strcat(jStr, v);

	return jStr;
}

char *addJSONStr(char *jStr, char *k, char *v)
{
	int nj = 0;

	nj = (int)strlen(jStr);
	if (nj == 2) {
		jStr[strlen(jStr) - 1] = '\0';
		//jSize--;
	}
	if (nj > 2) {
		jStr[strlen(jStr) - 1] = ',';
	}
	strcat(jStr, "\"");
	strcat(jStr, k);
	strcat(jStr, "\":");

	strcat(jStr, "\"");
	strcat(jStr, v);
	strcat(jStr, "\"");
	strcat(jStr, "}");

	return jStr;
}


static char encoding_table[] = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
				 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
				 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
				 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
				 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
				 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
				 'w', 'x', 'y', 'z', '0', '1', '2', '3',
				 '4', '5', '6', '7', '8', '9', '+', '/' };
static int mod_table[] = { 0, 2, 1 };


char *base64_encode(const unsigned char *data, size_t input_length, size_t *output_length)
{
	*output_length = 4 * ((input_length + 2) / 3) + 2;

	char *encoded_data = malloc(*output_length);
	if (encoded_data == NULL) return NULL;

	int i, j;
	for (i = 0, j = 0; i < input_length; ) {
		uint32_t octet_a = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t octet_b = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t octet_c = i < input_length ? (unsigned char)data[i++] : 0;
		uint32_t triple = (octet_a << 0x10) + (octet_b << 0x08) + octet_c;

		encoded_data[j++] = encoding_table[(triple >> 3 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 2 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 1 * 6) & 0x3F];
		encoded_data[j++] = encoding_table[(triple >> 0 * 6) & 0x3F];
	}

	for (i = 0; i < mod_table[input_length % 3]; i++) encoded_data[*output_length - 1 - i] = '=';

	encoded_data[*output_length - 1] = (char)"=";
	encoded_data[*output_length - 2] = (char)"=";

	return encoded_data;
}

char *compose_image_header(char *json, char *msgId, int total_imgs)
{

	char count[10];

	sprintf(count, "%d", total_imgs);

	json = addJSONStr(json, "result", "success");
	json = addJSONStr(json, "reason", "1");
	json = addJSONStr(json, "count", count);
	json = addJSONStr(json, "msgId", msgId);

	return json;
}

char *compose_downlscaled_image(char *main_json, char *image_buffer, int image_buffer_size, int img_id, char *img_name, int w, int h, gboolean is_first_item)
{

	char *list_json = NULL;
	char image_size[100], sw[10], sh[10];
	char simg_id[10];

	sprintf(image_size, "%d", image_buffer_size);
	sprintf(sw, "%d", w);
	sprintf(sh, "%d", h);
	sprintf(simg_id, "%d", img_id);

	list_json = allocate_json(image_buffer_size + 4096);

	list_json = addJSONStr(list_json, "id", simg_id);
	list_json = addJSONStr(list_json, "name", img_name);
	list_json = addJSONStr(list_json, "height", sw);
	list_json = addJSONStr(list_json, "width", sh);
	list_json = addJSONStr(list_json, "size", image_size);
	list_json = addJSONStr(list_json, "image", image_buffer);

	if (is_first_item == 1) {
		main_json = addJSONListStr(main_json, "list", list_json);
	} else {
		strcat(main_json, ",");
		strcat(main_json, list_json);
	}

	free(list_json);
	return main_json;
}

char *compose_fullscale_image(char *main_json, char *image_buffer, int image_buffer_size, int img_id, char *img_name, int width, int height)
{

	char *list_json = NULL;

	char image_size[100], swidth[10], sheight[10], simg_id[5];
	sprintf(image_size, "%d", image_buffer_size);

	sprintf(swidth, "%d", width);
	sprintf(sheight, "%d", height);
	sprintf(simg_id, "%d", img_id);

	list_json = allocate_json(image_buffer_size + 4096);

	list_json = addJSONStr(list_json, "id", simg_id);
	list_json = addJSONStr(list_json, "name", img_name);
	list_json = addJSONStr(list_json, "height", swidth);
	list_json = addJSONStr(list_json, "width", sheight);
	list_json = addJSONStr(list_json, "size", image_size);
	list_json = addJSONStr(list_json, "image", image_buffer);

	main_json = addImageJSON(main_json, "image", list_json);
	strcat(main_json, "}");

	free(list_json);

	return main_json;
}

char *allocate_json(int size)
{

	char *jStr;

	jStr = (char *)malloc(size + 1);

	memset(jStr, 0, size + 1);

	if (NULL == jStr) {
		return 0;
	}

	*(jStr + 0) = '{';
	*(jStr + 1) = '}';
	*(jStr + 2) = '\0';

	return jStr;
}

